from rest_framework import serializers

import drf_file_upload.fields
from drf_file_upload import serializers as dfu_serializers
from tests import models as test_models


class TestUserFileUploadSerializer(dfu_serializers.UploadedFileModelSerializer):
    user = serializers.HiddenField(default=serializers.CurrentUserDefault())
    file = drf_file_upload.fields.UploadedFileField(required=True, allow_null=False)

    class Meta:
        model = test_models.TestUserFileUpload
        fields = ("user", "file", "created")


class TestAnonFileUploadSerializer(dfu_serializers.UploadedFileModelSerializer):
    file = drf_file_upload.fields.AnonymousUploadedFileField(
        required=True, allow_null=False
    )

    class Meta:
        model = test_models.TestAnonFileUpload
        fields = ("file", "created")


class TestUserFileMetadataUploadSerializer(dfu_serializers.UploadedFileModelSerializer):
    metadata_uploaded_files_mapping = {
        "file": [("file", "file"), ("size", "meta_size"), ("name", "meta_name")]
    }

    user = serializers.HiddenField(default=serializers.CurrentUserDefault())
    file = drf_file_upload.fields.MetadataUploadedFileField(
        required=True, allow_null=False
    )
    meta_size = serializers.IntegerField(required=False, allow_null=True)
    meta_name = serializers.CharField(required=False, allow_null=True)

    class Meta:
        model = test_models.TestMetadataUserFileUpload
        fields = ("user", "file", "created", "meta_size", "meta_name")



import uuid
from abc import ABC, abstractmethod

# TODO: Test these


class BaseFilenameProvider(ABC):
    @abstractmethod
    def get_filename(self, original_filename, user=None):
        pass


class OriginalFilenameProvider(BaseFilenameProvider):
    def get_filename(self, original_filename, user=None):
        return original_filename


class UUIDFilenameProvider(BaseFilenameProvider):
    def get_filename(self, original_filename, user=None):
        ext = original_filename.split(".")[-1]
        return "%s.%s" % (uuid.uuid4(), ext)



# Generated by Django 3.2 on 2021-04-17 15:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("drf_file_upload", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="authenticateduploadedfile",
            name="uuid",
            field=models.CharField(default=None, max_length=64, null=True, unique=True),
        ),
    ]



# Generated by Django 3.2 on 2022-07-27 13:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("drf_file_upload", "0003_auto_20210420_2323"),
    ]

    operations = [
        migrations.AddField(
            model_name="anonymousuploadedfile",
            name="name",
            field=models.CharField(editable=False, max_length=254, null=True),
        ),
        migrations.AddField(
            model_name="anonymousuploadedfile",
            name="size",
            field=models.PositiveBigIntegerField(editable=False, null=True),
        ),
        migrations.AddField(
            model_name="authenticateduploadedfile",
            name="name",
            field=models.CharField(editable=False, max_length=254, null=True),
        ),
        migrations.AddField(
            model_name="authenticateduploadedfile",
            name="size",
            field=models.PositiveBigIntegerField(editable=False, null=True),
        ),
    ]



import mimetypes

from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from drf_file_upload import models
from drf_file_upload.fields import (
    AnonymousUploadedFileField,
    UploadedFileField,
    MetadataUploadedFileField,
)
from drf_file_upload.settings import lib_settings


class UploadFileValidationMixin:
    def validate_file(self, value):
        if not self.is_supported_file(value.name):
            raise ValidationError("invalid-file-format")
        if not self.respects_filesize_limit(value.size):
            raise ValidationError("file-too-large")
        return value

    def is_supported_file(self, file):
        allowed_formats = lib_settings.ALLOWED_FORMATS

        if not allowed_formats:
            return True

        mimetype = mimetypes.guess_type(file)[0]
        return mimetype in allowed_formats

    def respects_filesize_limit(self, size):
        max_file_size = lib_settings.MAX_FILE_SIZE
        if not max_file_size:
            return True
        return size <= max_file_size


class AuthenticatedUploadFileSerializer(
    UploadFileValidationMixin, serializers.ModelSerializer
):
    user = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = models.AuthenticatedUploadedFile
        fields = ["file", "user", "uuid", "size", "name"]
        read_only_fields = ["uuid", "user", "size", "name"]


class AnonymousUploadFileSerializer(
    UploadFileValidationMixin, serializers.ModelSerializer
):
    class Meta:
        model = models.AnonymousUploadedFile
        fields = ["file", "uuid", "size", "name"]
        read_only_fields = ["uuid", "size", "name"]


class UploadedFileSerializerMixin:
    metadata_uploaded_files_mapping = None

    """
    Example:

    metadata_uploaded_files_mapping = {
        "file": [("file", "file"), ("size", "size"), ("name", "name")]
    }
    """

    def _is_anon_field(self, field_type):
        return isinstance(field_type, AnonymousUploadedFileField)

    def _is_auth_field(self, field_type):
        return isinstance(field_type, UploadedFileField) or isinstance(
            field_type, MetadataUploadedFileField
        )

    def clean_uploaded_files(self, validated_data):
        for field_name, field_type in self.get_fields().items():
            if (self._is_anon_field(field_type) and field_name in validated_data
                and validated_data[field_name] is not None):
                instance = models.AnonymousUploadedFile.objects.get(
                    file=validated_data[field_name]
                )
                instance.delete(keep_file=True)
            if (self._is_auth_field(field_type) and field_name in validated_data
                and validated_data[field_name] is not None):
                instance = models.AuthenticatedUploadedFile.objects.get(
                    file=validated_data[field_name]
                )
                instance.delete(keep_file=True)

    def save(self, **kwargs):
        self.clean_uploaded_files(self.validated_data)
        return super().save(**kwargs)

    def validate(self, attrs):
        if isinstance(self.metadata_uploaded_files_mapping, dict):
            self.map_metadata_file_uploads(attrs)
        return attrs

    def map_metadata_file_uploads(self, attrs):
        for mapped_field, mappings in self.metadata_uploaded_files_mapping.items():
            value = attrs.get(mapped_field)
            if value:
                for src_mapping, dst_mapping in mappings:
                    attrs[dst_mapping] = value[src_mapping]


class UploadedFileSerializer(UploadedFileSerializerMixin, serializers.Serializer):
    pass


class UploadedFileModelSerializer(
    UploadedFileSerializerMixin, serializers.ModelSerializer
):
    pass


class UploadFileRequestSerializer(serializers.Serializer):
    file = serializers.FileField()



import os
import re

from rest_framework import status
from rest_framework.test import APIClient

from drf_file_upload import models
from tests.tests import factory
from tests.tests.base_test import BaseDrfFileUploadTestCase

API_ENDPOINT = "/upload/"


class AuthenticatedFileUploadTestCase(BaseDrfFileUploadTestCase):
    def setUp(self):
        super().setUp()
        self.client = APIClient()
        self.user = factory.create_user()

    def test_file_upload_requires_logged_user(self):
        response = self.upload_file(authenticate=False)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

        self.assertEqual(models.AuthenticatedUploadedFile.objects.count(), 0)

    def test_logged_user_can_upload_file(self):
        response = self.upload_file()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        response_data = response.json()
        self.assertTrue("uuid" in response_data)
        self.assertTrue("file" in response_data)
        self.assertTrue(
            re.match(r"\w{8}-\w{4}-\w{4}-\w{4}-\w{12}", response_data["uuid"])
        )
        self.assertTrue(response_data["file"].startswith("http"))

        auth_uploaded_file = models.AuthenticatedUploadedFile.objects.get(
            uuid=response_data["uuid"]
        )
        self.assertIsNotNone(auth_uploaded_file.file)
        self.assertEqual(auth_uploaded_file.user, self.user)

        self.assertTrue(os.path.exists(auth_uploaded_file.file.path))

    def test_upload_filename_with_uuid_provider(self):
        with self.settings(
            REST_FRAMEWORK_FILE_UPLOAD={
                "AUTH_FILENAME_PROVIDER": "drf_file_upload.filenames.UUIDFilenameProvider"
            }
        ):
            response = self.upload_file()
            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

            response_data = response.json()
            self.assertTrue("uuid" in response_data)
            self.assertTrue("file" in response_data)
            self.assertTrue(
                re.match(r"\w{8}-\w{4}-\w{4}-\w{4}-\w{12}", response_data["uuid"])
            )
            self.assertTrue(response_data["file"].startswith("http"))

            auth_uploaded_file = models.AuthenticatedUploadedFile.objects.get(
                uuid=response_data["uuid"]
            )
            self.assertIsNotNone(auth_uploaded_file.file)
            self.assertEqual(auth_uploaded_file.user, self.user)

            self.assertTrue(os.path.exists(auth_uploaded_file.file.path))

    def test_upload_file_max_size_returns_error_with_file_too_large(self):
        with self.settings(REST_FRAMEWORK_FILE_UPLOAD={"MAX_FILE_SIZE": 100}):
            response = self.upload_file()
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
            error = response.json()
            self.assertTrue("file" in error)
            self.assertEqual(error["file"], ["file-too-large"])

    def test_upload_file_max_size(self):
        with self.settings(REST_FRAMEWORK_FILE_UPLOAD={"MAX_FILE_SIZE": 100000}):
            response = self.upload_file()
            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_upload_file_allowed_type_returns_error_with_invalid_type(self):
        with self.settings(
            REST_FRAMEWORK_FILE_UPLOAD={"ALLOWED_FORMATS": {"image/png", "image/jpg"}}
        ):
            response = self.upload_file(filename="invalid.pdf")

            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
            error = response.json()
            self.assertTrue("file" in error)
            self.assertEqual(error["file"], ["invalid-file-format"])

    def test_upload_file_allowed_type(self):
        with self.settings(
            REST_FRAMEWORK_FILE_UPLOAD={
                "ALLOWED_FORMATS": {"image/png", "image/jpg", "application/pdf"}
            }
        ):
            response = self.upload_file(filename="valid.pdf")

            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_delete_uploaded_file(self):
        self.client.force_authenticate(self.user)

        uploaded_file = factory.create_authenticated_uploaded_file(self.user)

        response = self.client.delete(f"{API_ENDPOINT}{uploaded_file.uuid}/")
        self.assertEquals(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(os.path.exists(uploaded_file.file.path))

    def upload_file(self, filename="test_file.pdf", authenticate=True):
        if authenticate:
            self.client.force_authenticate(self.user)
        file = factory.create_simple_uploaded_file(filename=filename)
        response = self.client.post(API_ENDPOINT, {"file": file}, format="multipart")
        return response



from django.urls import include, path
from rest_framework.routers import DefaultRouter

from tests import views

router = DefaultRouter()
router.register(
    r"test-upload-auth", views.TestUserFileUpload, basename="test_upload_auth"
)
router.register(
    r"test-upload-auth-meta",
    views.TestUserMetadataFileUpload,
    basename="test_upload_auth_meta",
)

urlpatterns = [
    path("", include("drf_file_upload.urls")),
] + router.urls



from rest_framework import viewsets
from rest_framework.parsers import JSONParser
from rest_framework.permissions import IsAuthenticated

from tests import models, serializers


class TestUserFileUpload(viewsets.ModelViewSet):
    serializer_class = serializers.TestUserFileUploadSerializer
    permission_classes = [IsAuthenticated]
    queryset = models.TestUserFileUpload.objects.all()

    parser_classes = [JSONParser]


class TestUserMetadataFileUpload(viewsets.ModelViewSet):
    serializer_class = serializers.TestUserFileMetadataUploadSerializer
    permission_classes = [IsAuthenticated]
    queryset = models.TestMetadataUserFileUpload.objects.all()

    parser_classes = [JSONParser]



import os
import uuid

from django.conf import settings
from django.core.files.storage import default_storage
from django.db import models
from django.dispatch import receiver

from drf_file_upload.settings import lib_settings


def get_authenticated_uploaded_file_path(instance, filename):
    filename_provider_class = lib_settings.AUTH_FILENAME_PROVIDER
    base_dir = lib_settings.AUTH_MEDIA_DIR

    filename_provider = filename_provider_class()
    filename = filename_provider.get_filename(filename, user=instance.user)

    return os.path.join(base_dir, filename)


def get_anonymous_uploaded_file_path(instance, filename):
    filename_provider_class = lib_settings.ANON_FILENAME_PROVIDER
    base_dir = lib_settings.ANON_MEDIA_DIR

    filename_provider = filename_provider_class()
    filename = filename_provider.get_filename(filename)

    return os.path.join(base_dir, filename)


class AbstractUploadedFile(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    uuid = models.UUIDField(unique=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=254, null=True, editable=False)
    size = models.PositiveBigIntegerField(null=True, editable=False)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name

    def save(self, **kwargs):
        self.name = self.file.name
        self.size = self.file.size
        return super().save(**kwargs)

    def delete(self, *args, **kwargs):
        keep_file = kwargs.pop("keep_file", False)
        if keep_file:
            self.file = None
        return super().delete(*args, **kwargs)


class AuthenticatedUploadedFile(AbstractUploadedFile):
    file = models.FileField(
        blank=False, null=False, upload_to=get_authenticated_uploaded_file_path
    )
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)


class AnonymousUploadedFile(AbstractUploadedFile):
    file = models.FileField(
        blank=False, null=False, upload_to=get_anonymous_uploaded_file_path
    )


def delete_file_if_exists(file):
    if file:
        file.delete(save=False)

def delete_file_on_change(model, instance):
    if not instance.pk:
        return False

    try:
        old_file = model.objects.get(pk=instance.pk).file
    except model.DoesNotExist:
        return False

    new_file = instance.file
    if old_file and old_file != new_file:
        delete_file_if_exists(old_file)


@receiver(models.signals.post_delete, sender=AuthenticatedUploadedFile)
def delete_files_on_auth_uploaded_file_delete(sender, instance, **kwargs):
    delete_file_if_exists(instance.file)


@receiver(models.signals.pre_save, sender=AuthenticatedUploadedFile)
def delete_files_on_auth_uploaded_file_change(sender, instance, **kwargs):
    delete_file_on_change(AuthenticatedUploadedFile, instance)


@receiver(models.signals.post_delete, sender=AnonymousUploadedFile)
def delete_files_on_anon_uploaded_file_delete(sender, instance, **kwargs):
    delete_file_if_exists(instance.file)


@receiver(models.signals.pre_save, sender=AnonymousUploadedFile)
def delete_files_on_anon_uploaded_file_change(sender, instance, **kwargs):
    delete_file_on_change(AnonymousUploadedFile, instance)



# Generated by Django 3.2 on 2021-04-20 21:23

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("drf_file_upload", "0002_authenticateduploadedfile_uuid"),
    ]

    operations = [
        migrations.AlterField(
            model_name="anonymousuploadedfile",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
        ),
        migrations.AlterField(
            model_name="authenticateduploadedfile",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
        ),
    ]



from drf_spectacular.utils import extend_schema
from rest_framework import mixins
from rest_framework.parsers import FileUploadParser, MultiPartParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import GenericViewSet

from drf_file_upload import models, serializers


class AuthenticatedFileUploadView(
    mixins.CreateModelMixin, mixins.DestroyModelMixin, GenericViewSet
):
    parser_class = [FileUploadParser, MultiPartParser]
    permission_classes = [IsAuthenticated]
    serializer_class = serializers.AuthenticatedUploadFileSerializer
    lookup_field = "uuid"

    def get_queryset(self):
        return models.AuthenticatedUploadedFile.objects.filter(
            user_id=self.request.user.id
        )

    @extend_schema(
        request=serializers.UploadFileRequestSerializer,
        responses={200: serializers.AuthenticatedUploadFileSerializer},
    )
    def create(self, request, *args, **kwargs):
        return super().create(request, *args, **kwargs)


class AnonymousFileUploadView(
    mixins.CreateModelMixin, mixins.DestroyModelMixin, GenericViewSet
):
    parser_class = [FileUploadParser, MultiPartParser]
    permission_classes = []
    serializer_class = serializers.AnonymousUploadFileSerializer
    lookup_field = "uuid"

    def get_queryset(self):
        return models.AnonymousUploadedFile.objects.all()

    @extend_schema(
        request=serializers.UploadFileRequestSerializer,
        responses={200: serializers.AnonymousUploadFileSerializer},
    )
    def create(self, request, *args, **kwargs):
        return super().create(request, *args, **kwargs)



import logging
from datetime import timedelta

from django.utils import timezone

from drf_file_upload import models

logger = logging.getLogger("DrfFileUploadExpiredFiles")


def cleanup_expired_uploaded_files(expiration_seconds):
    delete_before = timezone.now() - timedelta(seconds=expiration_seconds)
    deleted, _ = models.AuthenticatedUploadedFile.objects.filter(
        created__lt=delete_before
    ).delete()
    logger.info(f"Deleted {deleted} unused authenticated uploaded files")

    deleted, _ = models.AnonymousUploadedFile.objects.filter(
        created__lt=delete_before
    ).delete()
    logger.info(f"Deleted {deleted} unused anonymous uploaded files")



from django.conf import settings
from django.db import models


class TestUserFileUpload(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    file = models.FileField()
    created = models.DateTimeField(auto_now_add=True)


class TestAnonFileUpload(models.Model):
    file = models.FileField()
    created = models.DateTimeField(auto_now_add=True)


class TestMetadataUserFileUpload(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    file = models.FileField()
    created = models.DateTimeField(auto_now_add=True)
    meta_size = models.PositiveBigIntegerField()
    meta_name = models.CharField(max_length=255)
